
/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void SPI_init(void);
static void SPI_init_BB(void);


#define INSTR_WRITE 0
#define INSTR_READ  0x80

#define INSTR_FIFO  0
#define INSTR_MAP   0x40

void delay_us_timer6(unsigned short delay_us) {
	// This function wait for (at least) delay_us microseconds
	// it is fairly unprecise for values below 3.
	// This function will ALWAYS last longer than requested

	if (!delay_us) {                  // no need to wait if zero
		return;
	}
	if (delay_us > 2) {				  // this funcrion itself has a workload of approx 1µS
		delay_us = delay_us - 2;      // as we count from 0 to delay_us-1, we have to substract 2
	} else {
		delay_us = 1;			      // 1 is the minimal value, otherwise the timer is just not counting

	}
	RCC->APB1ENR = RCC_APB1ENR_TIM6EN;  // set clock and power to timer 6
	//TIM6->CR2  = 0;
	//TIM6->DIER = 0;
	TIM6->PSC  = PERIPH_FREQ/1000000;   // set prescaler to count microseconds
	TIM6->ARR  = delay_us;          // set set max count value
	TIM6->CR1  = 0x9; // ARPE = 0 / OPM = 1 / URS = 0 / UDIS = 0 / CEN = 1
	while ((TIM6->CR1 & 1)&&(delay_us)) {	}           // wait while timer is enabled
}

unsigned char SPI_exc_BB(unsigned char dataout) {
	// This function sends a data word with the SPI protocol, and
	// returns the value received from the slave.

	unsigned char datain = 0;

	unsigned int tmp;
	for (int i=0; i<8; i++) {
		tmp = (dataout >> 7) & 1; // bit 0 of tmp = bit 7 of dataout
		                          // this is the next bit to send
		tmp <<= SPI1_MOSI_Pnum;   // place the bit to send in front os the MOSI bit
		SPI1_MOSI_GPIO_Port->ODR = (SPI1_MOSI_GPIO_Port->ODR & ~SPI1_MOSI_Pin) | tmp;
									// update the value of MOSI bit
		                            // clear with mask 'pin' and OR with 'tmp'

		SPI1_SCK_GPIO_Port->BSRR = SPI1_SCK_Pin << 16; // SPI_SCK = 0

		dataout <<= 1;

		datain <<= 1;
		tmp = (SPI1_MISO_GPIO_Port->IDR >> SPI1_MISO_Pnum) & 1;
		datain |= tmp;


		SPI1_SCK_GPIO_Port->BSRR = SPI1_SCK_Pin;       // SPI_SCK = 1

	}
	tmp = (dataout >> 7) & 1;
	tmp <<= SPI1_MOSI_Pnum;
	SPI1_MOSI_GPIO_Port->ODR = (SPI1_MOSI_GPIO_Port->ODR & ~SPI1_MOSI_Pin) | tmp;

	SPI1_SCK_GPIO_Port->BSRR = SPI1_SCK_Pin << 16; // SPI_SCK = 0
	return datain;
}


unsigned char SPI_exc(unsigned char dataout) {
	// to complete
	// ...
}





int main() {
	SystemClock_Config();
	SPI_init_BB();
//	SPI_init();

	while (1) {

	}
}


static void SPI_init_BB(void) {

	// enable clock on peripherals
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIOAEN;  // for SCK, MISO, MOSI
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIOBEN;  // for SSn

	// configure GPIO for SPI1_SCK
	// set Medium speed
	SPI1_SCK_GPIO_Port->OSPEEDR &= ~(GPIO_OSPEEDER_OSPEEDR0 << (SPI1_SCK_Pnum*2));
	SPI1_SCK_GPIO_Port->OSPEEDR |=   GPIO_SPEED_FREQ_HIGH   << (SPI1_SCK_Pnum*2);
	// set Push-Pull output type
	SPI1_SCK_GPIO_Port->OTYPER &= ~(GPIO_OTYPER_OT_0 << SPI1_SCK_Pnum);
	SPI1_SCK_GPIO_Port->OTYPER |=   OUTPUT_PP        << SPI1_SCK_Pnum;
	// disable pull up
	SPI1_SCK_GPIO_Port->PUPDR &= ~(GPIO_PUPDR_PUPDR0 << (SPI1_SCK_Pnum*2));
	SPI1_SCK_GPIO_Port->PUPDR |=   GPIO_NOPULL       << (SPI1_SCK_Pnum*2);
	// set direction mode output
	SPI1_SCK_GPIO_Port->MODER &= ~(GPIO_MODER_MODER0 << (SPI1_SCK_Pnum*2));
	SPI1_SCK_GPIO_Port->MODER |=   MODE_OUTPUT       << (SPI1_SCK_Pnum*2);

	// configure GPIO for SPI1_MISO
	SPI1_MISO_GPIO_Port->PUPDR &= ~(GPIO_PUPDR_PUPDR0 << (SPI1_MISO_Pnum*2));
	SPI1_MISO_GPIO_Port->PUPDR |=   GPIO_NOPULL       << (SPI1_MISO_Pnum*2);
	// set direction mode to input
	SPI1_MISO_GPIO_Port->MODER &= ~(GPIO_MODER_MODER0 << (SPI1_MISO_Pnum*2));
	SPI1_MISO_GPIO_Port->MODER |=   MODE_INPUT        << (SPI1_MISO_Pnum*2);

	// configure GPIO for SPI1_MOSI
	// set Medium speed
	SPI1_MOSI_GPIO_Port->OSPEEDR &= ~(GPIO_OSPEEDER_OSPEEDR0 << (SPI1_MOSI_Pnum*2));
	SPI1_MOSI_GPIO_Port->OSPEEDR |=   GPIO_SPEED_FREQ_HIGH   << (SPI1_MOSI_Pnum*2);
	// set Push-Pull output type
	SPI1_MOSI_GPIO_Port->OTYPER &= ~(GPIO_OTYPER_OT_0 << SPI1_MOSI_Pnum);
	SPI1_MOSI_GPIO_Port->OTYPER |=   OUTPUT_PP        << SPI1_MOSI_Pnum;
	// disable pull up
	SPI1_MOSI_GPIO_Port->PUPDR &= ~(GPIO_PUPDR_PUPDR0 << (SPI1_MOSI_Pnum*2));
	SPI1_MOSI_GPIO_Port->PUPDR |=   GPIO_NOPULL       << (SPI1_MOSI_Pnum*2);
	// set direction mode output
	SPI1_MOSI_GPIO_Port->MODER &= ~(GPIO_MODER_MODER0 << (SPI1_MOSI_Pnum*2));
	SPI1_MOSI_GPIO_Port->MODER |=   MODE_OUTPUT       << (SPI1_MOSI_Pnum*2);

	// configure GPIO for SPI1_SSn
	// set Medium speed
	SPI1_SSn_GPIO_Port->OSPEEDR &= ~(GPIO_OSPEEDER_OSPEEDR0 << (SPI1_SSn_Pnum*2));
	SPI1_SSn_GPIO_Port->OSPEEDR |=   GPIO_SPEED_FREQ_HIGH   << (SPI1_SSn_Pnum*2);
	// set Push-Pull output type
	SPI1_SSn_GPIO_Port->OTYPER &= ~(GPIO_OTYPER_OT_0 << SPI1_SSn_Pnum);
	SPI1_SSn_GPIO_Port->OTYPER |=   OUTPUT_PP        << SPI1_SSn_Pnum;
	// disable pull up
	SPI1_SSn_GPIO_Port->PUPDR &= ~(GPIO_PUPDR_PUPDR0 << (SPI1_SSn_Pnum*2));
	SPI1_SSn_GPIO_Port->PUPDR |=   GPIO_NOPULL       << (SPI1_SSn_Pnum*2);
	// set direction mode output
	SPI1_SSn_GPIO_Port->MODER &= ~(GPIO_MODER_MODER0 << (SPI1_SSn_Pnum*2));
	SPI1_SSn_GPIO_Port->MODER |=   MODE_OUTPUT       << (SPI1_SSn_Pnum*2);

    SPI1_SSn_GPIO_Port->BSRR = SPI1_SSn_Pin; // Slave select = 1

}


static void SPI_init(void) {

	// enable clock on peripherals
	RCC->APB2ENR |= RCC_APB2ENR_SPI1EN;   // The SPI1 module itself
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIOBEN;  // for SPI I/Os


    // configure GPIO for SPI1_SCK
    SPI1_SCK_GPIO_Port->AFR[SPI1_SCK_Pnum/8] = 0;	// to complete
    SPI1_SCK_GPIO_Port->MODER                = 0;   // to complete

    // configure GPIO for SPI1_MISO
    SPI1_MISO_GPIO_Port->AFR[SPI1_MISO_Pnum/8] &= 0; // to complete
    SPI1_MISO_GPIO_Port->MODER                 &= 0; // to complete

    // configure GPIO for SPI1_MOSI
    SPI1_MOSI_GPIO_Port->AFR[SPI1_MOSI_Pnum/8] &= 0; // to complete
    SPI1_MOSI_GPIO_Port->MODER                 &= 0; // to complete

    // set SPI module configuration
	SPI1->CR1     = 0; // to complete
	SPI1->CR2     = 0; // to complete
	SPI1->I2SCFGR = 0; // to complete

}

/************************************************************************
 * code below is not to be considered for this lessons
 ************************************************************************/
void SystemClock_Config(void)
{
	__disable_irq();
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Configure the main internal regulator output voltage
  */
  __HAL_RCC_PWR_CLK_ENABLE();
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE3);

  /** Initializes the RCC Oscillators according to the specified parameters
  * in the RCC_OscInitTypeDef structure.
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
  RCC_OscInitStruct.PLL.PLLM = 16;
  RCC_OscInitStruct.PLL.PLLN = 336;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV4;
  RCC_OscInitStruct.PLL.PLLQ = 2;
  RCC_OscInitStruct.PLL.PLLR = 2;
  HAL_RCC_OscConfig(&RCC_OscInitStruct);
  /** Initializes the CPU, AHB and APB buses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2);

}

